<?php
/*
 * This file is part of Totara LMS
 *
 * Copyright (C) 2017 onwards Totara Learning Solutions LTD
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @copyright 2017 onwards Totara Learning Solutions LTD
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @author    Joby Harding <joby.harding@totaralearning.com>
 */

defined('MOODLE_INTERNAL') || die();

use \filter_pip\pip_parser;

class {theme_name}_format_topics_renderer extends format_topics_renderer { //TODO: change {theme_name}

    const BEM_PREFIX = 'course-format--topics__';

    /**
     * Collapse states for the current course only.
     *
     * @var stdClass
     */
    protected  $collapsestates = null;

    /**
     * Constructor method, calls the parent constructor
     *
     * @param moodle_page $page
     * @param string $target one of rendering target constants
     */
    public function __construct(moodle_page $page, $target) {
        global $PAGE, $COURSE;

        parent::__construct($page, $target);

        // JS module initialises once only even if the init() method is called multiple times.
        $jsargs = [$COURSE->id, $this->get_collapsestates($COURSE->id)];
        user_preference_allow_ajax_update($this->topics_collapse_states_preference_name($COURSE->id), PARAM_RAW);
        $PAGE->requires->js_call_amd('theme_basis/format-topics', 'init', $jsargs);
    }

    /**
     * Generate user preference name
     *
     * @param int $courseid
     * @return string
     */
    protected function topics_collapse_states_preference_name($courseid) {
        return "theme_basis/topics_collapse_state--{$courseid}";
    }

    /**
     * Get collapsestates as stdClass prepopulating not yet set.
     *
     * @param int $courseid
     * @return stdClass
     */
    protected function get_collapsestates($courseid) {

        // Static cache.
        if ($this->collapsestates !== null) {
            return $this->collapsestates;
        }

        $this->collapsestates = json_decode(get_user_preferences($this->topics_collapse_states_preference_name
        ($courseid)));

        // Deal with an unset preference.
        if ($this->collapsestates === null) {
            $this->collapsestates = new stdClass();
        }

        return $this->collapsestates;

    }

    public function section_title($section, $course) {
        global $OUTPUT, $PAGE;

        if ($PAGE->user_is_editing()) {
            return parent::section_title($section, $course);
        }

        $prefix = self::BEM_PREFIX;
        $id = "#{$prefix}topic--{$section->section}";
        $attributes = [
            'data-toggle' => 'collapse',
            'class' => "{$prefix}topic__collapse-link",
            'aria-expanded' => ($this->section_is_collapsed($section) === true) ? 'false' : 'true',
            'aria-controls' => $id,
        ];

        // First section is open by default all others collapsed.
        $collapseicon = $OUTPUT->flex_icon('caret-down', ['classes' => "{$prefix}topic__collapse-link__icon"]);
        $collapselink = html_writer::link($id, $collapseicon, $attributes);

        return parent::section_title($section, $course) . $collapselink;
    }


    /**
     * Overrides parent method.
     *
     * Due to lack of encapsulation in the original method we have to
     * copy all this logic just to add a <div> start tag.
     */
    protected function section_header($section, $course, $onsectionpage, $sectionreturn=null) {
        global $PAGE;

        //
        // Overridden renderer additions.
        //
        $prefix = self::BEM_PREFIX;

        $rawsummarytext = $section->summary;

        $parser = (new pip_parser())
            ->add($rawsummarytext);

        $properties = $parser->parse();

        $headerclasses = " {$prefix}topic__header";

        //
        // END Overridden renderer additions.
        //

        $o = '';
        $currenttext = '';
        $sectionstyle = '';

        if ($section->section != 0) {
            // Only in the non-general sections.
            if (!$section->visible) {
                $sectionstyle = ' hidden';
            } else if (course_get_format($course)->is_section_current($section)) {
                $sectionstyle = ' current';
            }
        }

        $liclasses = ['section', 'main', 'clearfix', trim($sectionstyle)];
        if ($section->section !== 0) {
            // First section is the 'general' section containing coursewide
            // forum etc. so isn't really a topic.
            $liclasses[] = "{$prefix}topic";
        }
        $o.= html_writer::start_tag('li', array(
            'id' => 'section-'.$section->section,
            'class' => implode(' ', $liclasses),
            'role'=>'region',
            'aria-label'=> get_section_name($course, $section),
            'data-{theme_name}-course-section-id' => $section->id, //TODO: change {theme_name}
            'data-{theme_name}-is-collapsed' => ($this->section_is_collapsed($section) ? '1' : '0'), //TODO: change {theme_name}
        ));

        // Create a span that contains the section title to be used to create the keyboard section move menu.
        $o .= html_writer::tag('span', get_section_name($course, $section), array('class' => "hidden sectionname"));

        $leftcontent = $this->section_left_content($section, $course, $onsectionpage);
        $o.= html_writer::tag('div', $leftcontent, array('class' => 'left side'));

        $rightcontent = $this->section_right_content($section, $course, $onsectionpage);
        $o.= html_writer::tag('div', $rightcontent, array('class' => 'right side'));
        $o.= html_writer::start_tag('div', array('class' => 'content'));

        // When not on a section page, we display the section titles except the general section if null
        $hasnamenotsecpg = (!$onsectionpage && ($section->section != 0 || !is_null($section->name)));

        // When on a section page, we only display the general section title, if title is not the default one
        $hasnamesecpg = ($onsectionpage && ($section->section == 0 && !is_null($section->name)));

        $classes = ' accesshide';
        if ($hasnamenotsecpg || $hasnamesecpg) {
            $classes = '';
        }

        $sectiontitle = $this->section_title($section, $course);
        $sectionname = html_writer::tag('span', $sectiontitle, array('data-movetext' => 'true'));
        $clearfix = html_writer::span('', 'clearfix');
        $o.= html_writer::tag('h3', $sectionname . $clearfix, array(
            'class' => renderer_base::prepare_classes('sectionname' . $classes . $headerclasses),
            'data-toggle' => 'collapse',
            'data-target' => "#{$prefix}topic--{$section->section}",
        ));

        //
        // Overridden renderer additions.
        //
        $classes = "{$prefix}topic__content";

        // Only first section expanded by default.
        if (!$PAGE->user_is_editing()) {

            // Bootstrap 3 collapse.
            $classes .= ' collapse';
            if (!$this->section_is_collapsed($section)) {
                $classes .= ' in';
            }

            $o .= html_writer::start_div($classes, ['id' => "{$prefix}topic--{$section->section}"]);

        }
        //
        // END Overridden renderer additions.
        //

        $o.= html_writer::start_tag('div', array('class' => 'summary'));
        $o.= $this->format_summary_text($section);
        $o.= html_writer::end_tag('div');

        $context = context_course::instance($course->id);
        $o .= $this->section_availability_message($section,
            has_capability('moodle/course:viewhiddensections', $context));

        return $o;
    }

    protected function section_footer() {
        global $PAGE;

        $output = '';

        if (!$PAGE->user_is_editing()) {
            // End div.{self::BEM_PREFIX}topic__content opened in section_header() above.
            $output .= html_writer::end_div();
        }

        $output .= parent::section_footer();

        return $output;
    }

    /**
     * Get the current collapse state of the section.
     *
     * @param int $sectionid
     * @return boolean
     */
    public function section_is_collapsed($section, $courseid = null) {

        global $COURSE;

        if ($courseid === null) {
            $courseid = $COURSE->id;
        }

        // The course wide section is not a topic.
        if ($section->section === 0) {
            return false;
        }

        $coursecollapsestates = $this->get_collapsestates($courseid);

        // Is there already a user preference?
        if (isset($coursecollapsestates->{$section->id})) {
            return (int)$coursecollapsestates->{$section->id} === 1 ? true : false;
        }

        // Defaults - first topic expanded rest collapsed.
        if ($section->section === 1) {
            return false;
        }

        return true;
    }

    /**
     * Generate the title for this section page.
     *
     * @return string the page title
     */
    protected function page_title() {
        global $COURSE;

        $srtext = html_writer::span(' ' . get_string('topicoutline'), 'sr-only');
        $attributes = ['class' => self::BEM_PREFIX . 'header'];

        return html_writer::tag('h2', ($COURSE->fullname . $srtext), $attributes);
    }

    /**
     * Output the html for a multiple section page
     *
     * @param stdClass $course The course entry from DB
     * @param array $sections (argument not used)
     * @param array $mods (argument not used)
     * @param array $modnames (argument not used)
     * @param array $modnamesused (argument not used)
     */
    public function print_multiple_section_page($course, $sections, $mods, $modnames, $modnamesused) {
        global $PAGE;

        $modinfo = get_fast_modinfo($course);
        $course = course_get_format($course)->get_course();

        $context = context_course::instance($course->id);
        // Title with completion help icon.
        $completioninfo = new completion_info($course);
        echo $completioninfo->display_help_icon();

        //
        // Overridden renderer additions.
        //
        echo $this->page_title();
        //
        // END Overridden renderer additions.
        //

        // Copy activity clipboard..
        echo $this->course_activity_clipboard($course, 0);

        // Now the list of sections..
        echo $this->start_section_list();

        foreach ($modinfo->get_section_info_all() as $section => $thissection) {
            if ($section == 0) {
                // 0-section is displayed a little different then the others
                if ($thissection->summary or !empty($modinfo->sections[0]) or $PAGE->user_is_editing()) {
                    echo $this->section_header($thissection, $course, false, 0);
                    echo $this->courserenderer->course_section_cm_list($course, $thissection, 0);
                    echo $this->courserenderer->course_section_add_cm_control($course, 0, 0);
                    echo $this->section_footer();
                }
                continue;
            }
            if ($section > $course->numsections) {
                // activities inside this section are 'orphaned', this section will be printed as 'stealth' below
                continue;
            }
            // Show the section if the user is permitted to access it, OR if it's not available
            // but there is some available info text which explains the reason & should display.
            $showsection = $thissection->uservisible ||
                ($thissection->visible && !$thissection->available &&
                    !empty($thissection->availableinfo));
            if (!$showsection) {
                // If the hiddensections option is set to 'show hidden sections in collapsed
                // form', then display the hidden section message - UNLESS the section is
                // hidden by the availability system, which is set to hide the reason.
                if (!$course->hiddensections && $thissection->available) {
                    echo $this->section_hidden($section, $course->id);
                }

                continue;
            }

            if (!$PAGE->user_is_editing() && $course->coursedisplay == COURSE_DISPLAY_MULTIPAGE) {
                // Display section summary only.
                echo $this->section_summary($thissection, $course, null);
            } else {
                echo $this->section_header($thissection, $course, false, 0);
                if ($thissection->uservisible) {
                    echo $this->courserenderer->course_section_cm_list($course, $thissection, 0);
                    echo $this->courserenderer->course_section_add_cm_control($course, $section, 0);
                }
                echo $this->section_footer();
            }
        }

        if ($PAGE->user_is_editing() and has_capability('moodle/course:update', $context)) {
            // Print stealth sections if present.
            foreach ($modinfo->get_section_info_all() as $section => $thissection) {
                if ($section <= $course->numsections or empty($modinfo->sections[$section])) {
                    // this is not stealth section or it is empty
                    continue;
                }
                echo $this->stealth_section_header($section);
                echo $this->courserenderer->course_section_cm_list($course, $thissection, 0);
                echo $this->stealth_section_footer();
            }

            echo $this->end_section_list();

            echo html_writer::start_tag('div', array('id' => 'changenumsections', 'class' => 'mdl-right'));

            // Increase number of sections.
            $straddsection = get_string('increasesections', 'moodle');
            $url = new moodle_url('/course/changenumsections.php',
                array('courseid' => $course->id,
                    'increase' => true,
                    'sesskey' => sesskey()));
            $icon = $this->output->pix_icon('t/more', $straddsection);
            echo html_writer::link($url, $icon.get_accesshide($straddsection), array('class' => 'increase-sections'));

            if ($course->numsections > 0) {
                // Reduce number of sections sections.
                $strremovesection = get_string('reducesections', 'moodle');
                $url = new moodle_url('/course/changenumsections.php',
                    array('courseid' => $course->id,
                        'increase' => false,
                        'sesskey' => sesskey()));
                $icon = $this->output->pix_icon('t/less', $strremovesection);
                echo html_writer::link($url, $icon.get_accesshide($strremovesection), array('class' => 'reduce-sections'));
            }

            echo html_writer::end_tag('div');
        } else {
            echo $this->end_section_list();
        }

    }

}